//
//  FTLineProtocol.m
//  FTMobileSDKUnitTests
//
//  Created by hulilei on 2022/5/7.
//  Copyright © 2022 DataFlux-cn. All rights reserved.
//

#import <XCTest/XCTest.h>
#import <FTMobileAgent/FTMobileAgent.h>
#import <FTMobileAgent/FTMobileAgent+Private.h>
#import <FTDataBase/FTTrackerEventDBTool.h>
#import <FTRecordModel.h>
#import <FTMobileAgent/FTBaseInfoHandler.h>
#import <FTDateUtil.h>
#import <FTConstants.h>
#import <FTJSONUtil.h>
#import <NSString+FTAdd.h>
//#import "FTBaseInfoHander.h"
#import "FTTrackDataManger+Test.h"
#import <FTRequest.h>
#import <FTNetworkManager.h>
#import "FTRequestBody.h"
#import "FTModelHelper.h"
#import <FTEnumConstant.h>

@interface FTLineProtocol : XCTestCase

@end

@implementation FTLineProtocol

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}
- (void)testLineProtocol{
    NSDictionary *dict = @{
        FT_MEASUREMENT:@"iOSTest",
        FT_FIELDS:@{@"event":@"testLineProtocol"},
        FT_TAGS:@{@"name":@"testLineProtocol"},
    };
    NSDictionary *data =@{FT_AGENT_OP:FT_DATA_TYPE_RUM,
                          FT_AGENT_OPDATA:dict,
    };

    FTRecordModel *model = [FTRecordModel new];
    model.op =FT_DATA_TYPE_RUM;
    model.data =[FTJSONUtil convertToJsonData:data];
    FTRequestLineBody *line = [[FTRequestLineBody alloc]init];
    
    NSString *lineStr = [line getRequestBodyWithEventArray:@[model]];
    NSArray *array = [lineStr componentsSeparatedByString:@" "];
    XCTAssertTrue(array.count == 3);

    XCTAssertEqualObjects([array firstObject], @"iOSTest,name=testLineProtocol");
    XCTAssertEqualObjects(array[1], @"event=\"testLineProtocol\"");
    NSString *tm =[NSString stringWithFormat:@"%lld",model.tm];
    XCTAssertEqualObjects([array lastObject],tm);
}
- (void)testLogLineProtocol{
    FTRecordModel *model = [FTModelHelper createLogModel:@"testLogLineProtocol"];
    
    FTRequestLineBody *line = [[FTRequestLineBody alloc]init];
    
    NSString *lineStr = [line getRequestBodyWithEventArray:@[model]];
    //df_rum_ios_log,status=info message="testLogLineProtocol" 1651916317042660096
    NSArray *array = [lineStr componentsSeparatedByString:@" "];
    NSArray *sourceAndTagsArray = [[array firstObject] componentsSeparatedByString:@","];
    XCTAssertTrue([[sourceAndTagsArray firstObject] isEqualToString:FT_LOGGER_SOURCE]);
    XCTAssertTrue([sourceAndTagsArray containsObject:@"status=info"]);
    NSArray *fieldsArray = [array[1] componentsSeparatedByString:@","];
    XCTAssertTrue([fieldsArray containsObject:@"message=\"testLogLineProtocol\""]);
    NSString *times = [array lastObject];
    XCTAssertTrue([self isNum:times]);
}
- (void)testRumLineProtocol{
    NSDictionary *field = @{ FT_RUM_KEY_ERROR_MESSAGE:@"rum_model_create",
                             FT_RUM_KEY_ERROR_STACK:@"rum_model_create",
    };
    NSDictionary *tags = @{
        FT_RUM_KEY_ERROR_TYPE:@"ios_crash",
        FT_RUM_KEY_ERROR_SOURCE:@"logger",
        FT_RUM_KEY_ERROR_SITUATION:AppStateStringMap[AppStateRun],
        FT_RUM_KEY_SESSION_ID:[NSUUID UUID].UUIDString,
        FT_RUM_KEY_SESSION_TYPE:@"user",
    };
    long long time = [FTDateUtil currentTimeNanosecond];
    FTRecordModel *model = [[FTRecordModel alloc]initWithSource:FT_MEASUREMENT_RUM_ERROR op:FT_DATA_TYPE_RUM tags:tags field:field tm:time];
    
    FTRequestLineBody *line = [[FTRequestLineBody alloc]init];
    NSString *lineStr = [line getRequestBodyWithEventArray:@[model]];
    NSArray *array = [lineStr componentsSeparatedByString:@" "];
    NSArray *sourceAndTagsArray = [[array firstObject] componentsSeparatedByString:@","];
    XCTAssertTrue([[sourceAndTagsArray firstObject] isEqualToString:FT_MEASUREMENT_RUM_ERROR]);
    NSString *errorType = [NSString stringWithFormat:@"%@=%@",FT_RUM_KEY_ERROR_TYPE,@"ios_crash"];
    XCTAssertTrue([sourceAndTagsArray containsObject:errorType]);
    NSArray *fieldsArray = [array[1] componentsSeparatedByString:@","];
    NSString *errorMessage = [NSString stringWithFormat:@"%@=\"%@\"",FT_RUM_KEY_ERROR_MESSAGE,@"rum_model_create"];
    XCTAssertTrue([fieldsArray containsObject:errorMessage]);
    NSString *timeStr = [array lastObject];
    XCTAssertTrue([self isNum:timeStr] && [timeStr longLongValue] == time);
}
// message 实际为 "\"
- (void)testFieldValueHasTransliteration1{
    [self transliteration:@"\\" expect:@"\\\\"];
}
// message 实际为 "\\"
- (void)testFieldValueHasTransliteration2{
    [self transliteration:@"\\\\" expect:@"\\\\\\\\"];
}
// message 实际为 "\\\"
- (void)testFieldValueHasTransliteration3{
    [self transliteration:@"\\\\\\" expect:@"\\\\\\\\\\\\"];
}
- (void)testFieldValueJsonStr{
    NSDictionary *json = @{@"json":@"1",
                           @"json2":@"2"
    };
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:json options:NSJSONWritingPrettyPrinted error:&error];
    if (!error) {
        NSString *str  = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
        [self transliteration:str expect:@""];
    }
    
}
- (void)transliteration:(NSString *)str expect:(NSString *)expect{
    NSProcessInfo *processInfo = [NSProcessInfo processInfo];
    NSString *url = [processInfo environment][@"ACCESS_SERVER_URL"];
    [[FTTrackerEventDBTool sharedManger] deleteItemWithTm:[FTDateUtil currentTimeNanosecond]];
    FTMobileConfig *config = [[FTMobileConfig alloc]initWithMetricsUrl:url];
    FTLoggerConfig *loggerConfig = [[FTLoggerConfig alloc]init];
    loggerConfig.enableCustomLog = YES;
    [FTMobileAgent startWithConfigOptions:config];
    [[FTMobileAgent sharedInstance] startLoggerWithConfigOptions:loggerConfig];

    XCTestExpectation *expectation= [self expectationWithDescription:@"异步操作timeout"];
    FTRecordModel *model = [FTModelHelper createLogModel:str];
    FTRequestLineBody *line = [[FTRequestLineBody alloc]init];
    
    NSString *lineStr = [line getRequestBodyWithEventArray:@[model]];
    NSArray *array = [lineStr componentsSeparatedByString:@" "];
    if(array.count == 3){
        NSString *message = [NSString stringWithFormat:@"message=\"%@\"",expect];
        XCTAssertTrue([array[1] isEqualToString:message]);
    }
    FTRequest *request = [FTRequest createRequestWithEvents:@[model] type:FT_DATA_TYPE_LOGGING];
    [[FTNetworkManager sharedInstance] sendRequest:request completion:^(NSHTTPURLResponse * _Nonnull httpResponse, NSData * _Nullable data, NSError * _Nullable error) {
        if(!error){
        NSInteger statusCode = httpResponse.statusCode;
        BOOL success = (statusCode >=200 && statusCode < 500);
        XCTAssertTrue(success);
        }
        [expectation fulfill];
    }];
    [self waitForExpectationsWithTimeout:32 handler:^(NSError *error) {
    }];
}

- (BOOL)isNum:(NSString *)checkedNumString {
    checkedNumString = [checkedNumString stringByTrimmingCharactersInSet:[NSCharacterSet decimalDigitCharacterSet]];
    if(checkedNumString.length > 0) {
        return NO;
    }
    return YES;
}
@end
